package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dao.entity.BaseArea;
import com.abi.qms.platform.dto.res.GetBaseAreaListRes;
import com.abi.qms.platform.service.BaseAreaService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author: ZhangZhao
 * @date: 2021/7/16 16:21
 * @description: 省市-区域数据 Controller
 */
@RestController
@RequestMapping("/base/area")
@Api(tags = "省市-区域数据")
public class BaseAreaController {

    @Autowired
    private BaseAreaService areaService;

    /**
     * 根据区域类型获取区域信息列表 type:1-省(直辖市,特别行政区) 2-市 3-县(区)
     *
     * @param type 1-省(直辖市,特别行政区) 2-市 3-县(区)
     * @return GetBaseAreaListRes
     */
    @ApiOperation("根据区域类型获取区域信息列表 type:1-省(直辖市,特别行政区) 2-市 3-县(区)")
    @GetMapping(value = "/getAreaList/type/{type}")
    public BaseResponse<GetBaseAreaListRes> getAreaInfoListByType(@PathVariable("type") Integer type) {
        GetBaseAreaListRes areas = areaService.getAreaListByType(type);
        return BaseResponse.create(areas);
    }

    /**
     * 查询指定父级区域下所以子级区域信息
     *
     * @param parentId 父级区域Code
     * @return GetBaseAreaListRes
     */
    @ApiOperation("查询指定父级区域下所以子级区域信息")
    @GetMapping(value = "/getAreaList/parentCode/{code}")
    public BaseResponse<GetBaseAreaListRes> getAreaInfoListByParentCode(@PathVariable("code") Long parentId) {
        GetBaseAreaListRes areaList = areaService.getAreaListByParentId(parentId);
        return BaseResponse.create(areaList);
    }


}
