package com.abi.qms.platform.controller.console;

import com.abi.platform.Base.BaseResponse;
import com.abi.qms.platform.dto.req.GetBoxCodeReq;
import com.abi.qms.platform.dto.req.GetInvisibleCodeReq;
import com.abi.qms.platform.dto.req.GetStackCodeReq;
import com.abi.qms.platform.dto.res.GetBoxCodeRes;
import com.abi.qms.platform.dto.res.GetInvisibleCodeRes;
import com.abi.qms.platform.dto.res.GetStackCodeRes;
import com.abi.qms.platform.service.FlowQueryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 箱码垛码查询 Controller
 *
 * @author WeiganCai
 * @date 2021-08-02
 */
@Slf4j
@RestController
@Api(tags = "箱码垛码查询")
@RequestMapping("/flowQuery")
public class FlowQueryController {
    @Autowired
    private FlowQueryService flowQueryService;

    @ApiOperation("箱码查询")
    @GetMapping("/boxCode")
    public BaseResponse<GetBoxCodeRes> boxCode (@Validated GetBoxCodeReq getBoxCodeReq) {
        GetBoxCodeRes res = flowQueryService.boxCode(getBoxCodeReq);
        return BaseResponse.create(res);
    }

    @ApiOperation("隐形码模糊查询")
    @GetMapping("/invisibleCode/like")
    public BaseResponse<GetInvisibleCodeRes> invisibleCode (@Validated GetInvisibleCodeReq getInvisibleCodeReq) {
        GetInvisibleCodeRes res = flowQueryService.invisibleCode(getInvisibleCodeReq);
        return BaseResponse.create(res);
    }

    @ApiOperation("垛码查询")
    @GetMapping("/stackCode")
    public BaseResponse<GetStackCodeRes> stackCode(@Validated GetStackCodeReq getStackCodeReq) {
        GetStackCodeRes res = flowQueryService.stackCode(getStackCodeReq);
        return BaseResponse.create(res);
    }


}
