package com.abi.qms.platform.dto.req;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.List;

/**
 * @author: ZhangZhao
 * @date: 2021/7/19 14:20
 * @description: 新增稽查案件入参
 */
@Data
@NoArgsConstructor
@ApiModel
public class AddInspectionCaseReq implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 案件编号生成策略 1-自动生成 2-自定义
     */
    @NotNull(message = "案件编号生成策略不能为空")
    @ApiModelProperty(value = "案件编号生成策略 1-自动生成 2-自定义", required = true)
    private Integer caseNumberGenerateStrategy;

    /**
     * 省份id
     */
    @ApiModelProperty(value = "省份ID")
    private Long provinceId;


    /**
     * 城市id
     */
    @ApiModelProperty(value = "城市ID")
    private Long cityId;

    /**
     * 案件编号
     */
    @ApiModelProperty(value = "自定义-案件编号(自定义编号时必传)")
    private String caseNumber;

    /**
     * 案件名称
     */
    @NotBlank(message = "案件名称不能为空")
    @ApiModelProperty(value = "案件名称", required = true)
    private String caseName;

    /**
     * sku_id
     */
    @NotBlank(message = "sku_code不能为空")
    @ApiModelProperty(value = "sku_code", required = true)
    private String skuCode;


    /**
     * 案件备注
     */
    @Size(max = 150)
    @ApiModelProperty(value = "案件备注")
    private String caseRemark;

    /**
     * 案件涉及箱码列表
     */
    @Size(min = 1, message = "案件涉及箱码不能为空")
    @ApiModelProperty(value = "案件涉及箱码列表", required = true)
    private List<BoxCode> involvedBoxCodes;


    /**
     * 涉案箱码信息 BEAN
     */
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @ToString
    @ApiModel(value = "AddInspectionCaseReq_BoxCode")
    public static class BoxCode implements Serializable {

        /**
         * 箱码
         */
        @NotEmpty(message = "箱码不能为空")
        @Size(max = 32)
        @ApiModelProperty(value = "箱码", required = true)
        private String boxCode;

        /**
         * 手写标记
         */
        @Size(max = 15)
        @ApiModelProperty(value = "手写标记")
        private String handMark;

    }


}
