package com.abi.qms.platform.infrastructure.util;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.task.common.api.exception.BusinessException;

import java.util.Collection;
import java.util.function.Predicate;

/**
 * @Author AndyTan
 * @Description 异常断言util
 * @Date 2021/1/23
 **/
public class AssertUtil {

    /**
     * 如果是null，则抛出异常
     *
     * @param o
     * @param msg
     */
    public static void isNull(Object o, String msg) {
        if (o == null) {
            throw new BusinessException(msg);
        }
    }


    /**
     * 如果集合不为空，则抛出异常
     *
     * @param coll
     * @param msg
     */
    public static void isEmpty(Collection<?> coll, String msg) {
        if (CollectionUtil.isNotEmpty(coll)) {
            throw new BusinessException(msg);
        }
    }






    /**
     * 如果集合为空，则抛出异常
     *
     * @param coll
     * @param msg
     */
    public static void isNotEmpty(Collection<?> coll, String msg) {
        if (CollectionUtil.isEmpty(coll)) {
            throw new BusinessException(msg);
        }
    }


    /**
     * 如果目标值不满足当前表达式 抛出异常
     *
     * @param t         目标值
     * @param predicate 表达式
     * @param msg       异常信息
     * @param <T>       值类型
     */
    public static <T> void isMeets(T t, Predicate<T> predicate, String msg) {
        if (!predicate.test(t)) {
            throw new BusinessException(msg);
        }
    }
}
