package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.BaseArea;
import com.abi.qms.platform.dao.entity.BaseMaterial;
import com.abi.qms.platform.dao.mapper.BaseAreaMapper;
import com.abi.qms.platform.dto.res.GetBaseAreaListRes;
import com.abi.qms.platform.dto.res.ListBoxCodeFormatRes;
import com.abi.qms.platform.service.BaseAreaService;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author: ZhangZhao
 * @date: 2021/7/16 17:03
 * @description: 省市-区域数据服务层实现
 */
@Service
public class BaseAreaServiceImpl implements BaseAreaService {

    @Autowired
    private BaseAreaMapper areaMapper;

    /**
     * 根据区域类型查询区域信息
     *
     * @param type 区域类型
     * @return 区域信息列表
     */
    @Override
    public GetBaseAreaListRes getAreaListByType(Integer type) {
        QueryWrapper<BaseArea> queryWrapper = new QueryWrapper<BaseArea>();
        queryWrapper.eq("type", type);
        queryWrapper.eq("is_delete", 0);
        List<BaseArea> baseAreas = areaMapper.selectList(queryWrapper);
        List<GetBaseAreaListRes.BaseAreaBean> baseAreaBeans = PojoConverterUtils.copyList(baseAreas, GetBaseAreaListRes.BaseAreaBean.class);
        return new GetBaseAreaListRes(baseAreaBeans);
    }


    /**
     * 查询知道区域下所有子级区域信息
     *
     * @param parentId 父级标识
     * @return 区域信息列表
     */
    @Override
    public GetBaseAreaListRes getAreaListByParentId(Long parentId) {
        QueryWrapper<BaseArea> queryWrapper = new QueryWrapper<BaseArea>();
        queryWrapper.eq("parent_id", parentId);
        queryWrapper.eq("is_delete", 0);
        List<BaseArea> baseAreas = areaMapper.selectList(queryWrapper);
        List<GetBaseAreaListRes.BaseAreaBean> baseAreaBeans = PojoConverterUtils.copyList(baseAreas, GetBaseAreaListRes.BaseAreaBean.class);
        return new GetBaseAreaListRes(baseAreaBeans);
    }


}
