package com.abi.qms.platform.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.json.JSONUtil;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.enums.FlowQueryTypeEnum;
import com.abi.qms.platform.dao.enums.FlowStatusEnum;
import com.abi.qms.platform.dao.enums.ReqChannelTypeEnum;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.FlowQueryService;
import com.abi.qms.platform.service.IFlowQueryRecordService;
import com.abi.task.common.utils.PojoConverterUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 箱码垛码查询 Service业务层处理
 *
 * @author WeiganCai
 * @date 2021-08-02
 */
@Service
@Slf4j
public class FlowQueryServiceImpl implements FlowQueryService {


    @Autowired
    private IFlowQueryRecordService flowQueryRecordService;

    @Autowired
    private UserUtil userUtil;

    /**
     * 箱码查询
     * TODO 测试使用，待流向查询接口完成后，根据实际情况调整结构
     */
    @Override
    public GetBoxCodeRes boxCode(GetBoxCodeReq req) {
        Integer queryType = req.getQueryType();
        GetBoxCodeRes res = null;

        if (FlowQueryTypeEnum.BOX_CODE.is(queryType)) {
            // 箱码(明码)查询
            String boxCode = req.getBoxCode();
            res = this.getFlowInfoByBoxCode(boxCode);
        } else if (FlowQueryTypeEnum.INVISIBLE_CODE.is(queryType)) {
            // 箱码(隐形码)查询
            String boxCode = req.getBoxCode();
            res = this.getFlowInfoByBoxCode(boxCode);
        }

        AssertUtil.isNull(res, "未查询到流向信息！");
        //保存查询记录
        this.saveQueryRecord(res, req);
        return res;
    }

    /**
     * 查询流向信息
     *
     * @param boxCode 箱码
     * @return
     */
    @Override
    public GetBoxCodeRes getFlowInfoByBoxCode(String boxCode) {
        GetBoxCodeRes res = new GetBoxCodeRes();

        GetBoxCodeRes.SkuBean sku = new GetBoxCodeRes.SkuBean()
                .setBrandCode("01AAHF")
                .setBrandName("百威")
                .setSkuId("000000000000025000")
                .setSkuName("百威9.7度500ML1X18纸箱听装-温州版")
                .setProductionBatch("20210512CB0631HN-30")
                .setProductionTime(LocalDateTime.of(2021, 5, 21, 13, 0, 0));

        GetBoxCodeRes.InspectionCaseBean inspectionCase = new GetBoxCodeRes.InspectionCaseBean()
                .setCaseNumber("江苏省南京市#1")
                .setCaseName("模拟接口稽查案件");

        List flowList = new LinkedList();
        GetBoxCodeRes.FlowBean flow = new GetBoxCodeRes.FlowBean()
                .setNodeId(8902L)
                .setNodeName("CNA9-百威英博大富豪（盐城）啤酒有限公司")
                .setNodeStatus(1)
                .setIsSplit(0)
                .setOutStorageTime(LocalDateTime.of(2020, 12, 20, 9, 31, 33));
        flowList.add(flow);

        flow = new GetBoxCodeRes.FlowBean()
                .setNodeId(8903L)
                .setNodeName("上海吴泾DC")
                .setNodeStatus(1)
                .setIsSplit(1)
                .setSplitStackCode("275612348457ADB2")
                .setInStorageTime(LocalDateTime.of(2020, 12, 24, 11, 12, 38))
                .setOutStorageTime(LocalDateTime.of(2020, 12, 24, 15, 12, 33));
        flowList.add(flow);

        flow = new GetBoxCodeRes.FlowBean()
                .setNodeId(8904L)
                .setNodeName("一级经销商")
                .setNodeStatus(1)
                .setIsSplit(0)
                .setInStorageTime(LocalDateTime.of(2020, 12, 25, 11, 12, 38))
                .setOutStorageTime(LocalDateTime.of(2020, 12, 25, 15, 12, 33));
        flowList.add(flow);

        flow = new GetBoxCodeRes.FlowBean()
                .setNodeId(8905L)
                .setNodeName("二级经销商")
                .setNodeStatus(0)
                .setAbnormalType(2)
                .setIsSplit(1)
                .setSplitStackCode("200712348457ADB2")
                .setInStorageTime(LocalDateTime.of(2020, 12, 26, 11, 12, 38))
                .setOutStorageTime(LocalDateTime.of(2020, 12, 26, 15, 12, 33));
        flowList.add(flow);

        flow = new GetBoxCodeRes.FlowBean()
                .setNodeId(8906L)
                .setNodeName("杭州市延海百利超市（新华小区店）")
                .setNodeStatus(0)
                .setAbnormalType(1)
                .setIsSplit(0)
                .setInStorageTime(LocalDateTime.of(2020, 12, 27, 11, 12, 38));
        flowList.add(flow);
        Collections.reverse(flowList);

        GetBoxCodeRes.BoxStackBean boxStack = new GetBoxCodeRes.BoxStackBean()
                .setBoxCode(boxCode)
                .setInvisibleCode("adb25416313")
                .setOriginalStackCode("200700048457ADB2")
                .setOriginalStackTime(LocalDateTime.of(2020, 12, 26, 11, 12, 38))
                .setNewestStackCode("100733348457ADB2")
                .setNewestStackTime(LocalDateTime.of(2021, 1, 1, 11, 12, 38));


        res.setSku(sku);
        res.setInspectionCase(inspectionCase);
        res.setFlowList(flowList);
        res.setBoxStack(boxStack);
        return res;
    }



    /**
     * 隐形码查询
     * TODO 测试使用，待流向查询接口完成后，根据实际情况调整结构
     */
    @Override
    public GetInvisibleCodeRes invisibleCode(GetInvisibleCodeReq req) {
        String invisibleCode = req.getInvisibleCode();
        List<GetInvisibleCodeRes.InvisibleCodeBean> invisibleCodeList = new ArrayList<>();
        GetInvisibleCodeRes.InvisibleCodeBean invisibleCodeBean;
        // 模拟10条数据
        for (int i = 0; i < 10; i++) {
            invisibleCodeBean = new GetInvisibleCodeRes.InvisibleCodeBean()
                    .setBoxCode(invisibleCode.replace("_", String.valueOf(i)))
                    .setInvisibleCode(invisibleCode.replace("_", String.valueOf(i)))
                    .setStackTime(LocalDateTime.now());
            invisibleCodeList.add(invisibleCodeBean);
        }
        return new GetInvisibleCodeRes().setInvisibleCodeList(invisibleCodeList);
    }

    /**
     * 垛码查询
     * TODO 测试使用，待流向查询接口完成后，根据实际情况调整结构
     */
    @Override
    public GetStackCodeRes stackCode(GetStackCodeReq req) {
        GetStackCodeRes res = new GetStackCodeRes();

        GetStackCodeRes.SkuBean sku = new GetStackCodeRes.SkuBean()
                .setBrandCode("01AAHF")
                .setBrandName("百威")
                .setSkuId("000000000000025000")
                .setSkuName("百威9.7度330ML4X6半托纸箱听装（机械改包）");

        List flowList = new LinkedList();
        GetStackCodeRes.FlowBean flow = new GetStackCodeRes.FlowBean()
                .setNodeId(8903L)
                .setNodeName("一级经销商")
                .setInStorageTime(LocalDateTime.of(2020, 12, 24, 11, 12, 38))
                .setOutStorageTime(LocalDateTime.of(2020, 12, 24, 15, 12, 33));
        flowList.add(flow);

        flow = new GetStackCodeRes.FlowBean()
                .setNodeId(8902L)
                .setNodeName("XXXX-上海吴泾DC")
                .setInStorageTime(LocalDateTime.of(2020, 12, 20, 9, 31, 33))
                .setOutStorageTime(LocalDateTime.of(2020, 12, 20, 9, 31, 33));

        flowList.add(flow);
        Collections.reverse(flowList);

        GetStackCodeRes.StackBean stack = new GetStackCodeRes.StackBean()
                .setStackCode(req.getStackCode())
                .setStackTime(LocalDateTime.now());

        res.setSku(sku);
        res.setFlowList(flowList);
        res.setStack(stack);

        return res;
    }

    /**
     * 微信箱码查询
     */
    @Override
    public WxGetBoxCodeRes wxBoxCode(WxGetBoxCodeReq req) {
        // TODO 处理逻辑同console端，待流向查询接口完成后修改
        GetBoxCodeReq boxCodeReq = PojoConverterUtils.copy(req, GetBoxCodeReq.class);
        GetBoxCodeRes boxCodeRes = this.boxCode(boxCodeReq);
        return PojoConverterUtils.copy(boxCodeRes, WxGetBoxCodeRes.class);
    }


    /**
     * 微信隐形码查询
     */
    @Override
    public WxGetInvisibleCodeRes WxInvisibleCode(WxGetInvisibleCodeReq req) {
        // TODO 处理逻辑同console端，待流向查询接口完成后修改
        GetInvisibleCodeReq invisibleCodeReq = PojoConverterUtils.copy(req, GetInvisibleCodeReq.class);
        GetInvisibleCodeRes invisibleCodeRes = this.invisibleCode(invisibleCodeReq);
        return PojoConverterUtils.copy(invisibleCodeRes, WxGetInvisibleCodeRes.class);
    }

    /**
     * 微信垛码查询
     */
    @Override
    public WxGetStackCodeRes wxStackCode(WxGetStackCodeReq req) {
        // TODO 处理逻辑同console端，待流向查询接口完成后修改
        GetStackCodeReq stackCodeReq = PojoConverterUtils.copy(req, GetStackCodeReq.class);
        GetStackCodeRes stackCodeRes = this.stackCode(stackCodeReq);
        return PojoConverterUtils.copy(stackCodeRes, WxGetStackCodeRes.class);
    }

    /**
     * 保存查询记录
     *
     * @param res 查询结果
     * @param req 查询请求
     */
    private void saveQueryRecord(GetBoxCodeRes res, GetBoxCodeReq req) {
        try {

            //判断流向状态
            Map<Integer, GetBoxCodeRes.FlowBean> abnormalNodeMap = res.getFlowList()
                    .stream()
                    .filter(s -> !(s.getNodeStatus().equals(1)))
                    .collect(Collectors.toMap(GetBoxCodeRes.FlowBean::getAbnormalType, flowBean -> flowBean));


            FlowStatusEnum flowStatus = MapUtil.isEmpty(abnormalNodeMap) ? FlowStatusEnum.ABNORMAL : FlowStatusEnum.NORMAL;

            //查询类型
            Integer queryType = req.getQueryType();
            AddFlowQueryRecordReq addReq = new AddFlowQueryRecordReq();
            addReq.setFlowStatus(flowStatus.getCode());
            addReq.setQueryType(queryType);


            //  箱码 查询类型为箱码时必传
            addReq.setBoxCode(req.getBoxCode());
            addReq.setTrayCode("");

            //品牌
            addReq.setBrandCode(res.getSku().getBrandCode());
            addReq.setBrandName(res.getSku().getBrandName());

            //sku
            GetBoxCodeRes.SkuBean sku = res.getSku();
            addReq.setSkuCode(sku.getSkuId());
            addReq.setProductionBatch(sku.getProductionBatch());
            addReq.setProductionTime(sku.getProductionTime());


            //后台查询
            addReq.setQueryPlatform(ReqChannelTypeEnum.BACKSTAGE.getCode());

            //后台查询没有地址
            addReq.setQueryAddress("");

            //abnormalNodeIn
            GetBoxCodeRes.FlowBean abnormalNodeIn = abnormalNodeMap.get(1);
            if (Objects.nonNull(abnormalNodeIn)) {
                addReq.setAbnormalInNodeCode(abnormalNodeIn.getNodeId());
                addReq.setAbnormalInNodeName(abnormalNodeIn.getNodeName());
            }


            //abnormalNodeOut
            GetBoxCodeRes.FlowBean abnormalNodeOut = abnormalNodeMap.get(2);
            if (Objects.nonNull(abnormalNodeOut)) {
                addReq.setAbnormalOutNodeCode(abnormalNodeOut.getNodeId());
                addReq.setAbnormalOutNodeName(abnormalNodeOut.getNodeName());
            }
            //箱垛信息
            GetBoxCodeRes.BoxStackBean boxStack = res.getBoxStack();
            AddFlowQueryRecordReq.BoxStackBean boxStackBean = new AddFlowQueryRecordReq.BoxStackBean();
            BeanUtil.copyProperties(boxStack, boxStackBean);
            addReq.setBoxStackInfo(boxStackBean);

            //流向信息列表
            List<GetBoxCodeRes.FlowBean> flowList = res.getFlowList();
            List<AddFlowQueryRecordReq.FlowBean> flowBeans = PojoConverterUtils.copyList(flowList, AddFlowQueryRecordReq.FlowBean.class);
            addReq.setFlowList(flowBeans);

            flowQueryRecordService.addFlowQueryRecord(addReq);
        } catch (Exception ex) {
            //获取当前登录的用户
            UserInfo user = userUtil.getUser();
            log.error("查询流向信息记录报错失败！,查询人：【{}】,查询结果:【{}】,异常:{}", user.getUserName(), JSONUtil.toJsonStr(res), ex.getMessage());
        }
    }


}
