package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.InspectionCaseBoxCode;
import com.abi.qms.platform.dao.entity.InspectionCaseFlow;
import com.abi.qms.platform.dao.mapper.InspectionCaseFlowMapper;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.service.InspectionCaseFlowService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 稽查案件-流向关联 Service业务层处理
 *
 * @author WeiganCai
 * @date 2021-07-30
 */
@Service
@Slf4j
public class InspectionCaseFlowServiceImpl implements InspectionCaseFlowService {


    @Autowired
    private InspectionCaseFlowMapper inspectionCaseFlowMapper;

    /**
     * 保存【稽查案件-流向】关联关系
     * @param flowList 流向节点列表
     */
    @Override
    public void saveInspectionCaseFlowList( List<InspectionCaseFlow> flowList) {
        AssertUtil.isEmpty(flowList,"保存【稽查案件-流向】关联关系失败,流向信息为空！");
        inspectionCaseFlowMapper.insertBatch(flowList);
    }


    /**
     * 根据案件ID删除
     * @param caseId 案件ID
     */
    @Override
    public void deleteByCaseId(final Long caseId) {
        AssertUtil.isNull(caseId,"根据案件ID删除【稽查案件-流向】关联关系失败,案件ID为空！");
        QueryWrapper<InspectionCaseFlow> caseFlowQueryWrapper = new QueryWrapper<>();
        caseFlowQueryWrapper.eq("inspection_case_id", caseId);
        inspectionCaseFlowMapper.delete(caseFlowQueryWrapper);
    }
}
