package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.BaseArea;
import com.abi.qms.platform.dao.mapper.BaseAreaMapper;
import com.abi.qms.platform.dao.mapper.InspectionCaseNumberMapper;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.service.InspectionCaseNumberService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 稽查案件编号 Service业务层处理
 *
 * @author WeiganCai
 * @date 2021-07-20
 */
@Service
@Slf4j
public class InspectionCaseNumberServiceImpl implements InspectionCaseNumberService {
    @Autowired
    private InspectionCaseNumberMapper caseNumberMapper;

    @Autowired
    private BaseAreaMapper areaMapper;

    /**
     * 根据cityId获取案件编号
     */
    @Override
    public String getCaseNumber(Long cityId) {
        BaseArea city = areaMapper.selectById(cityId);
        AssertUtil.isNull(city, "城市不存在");

        BaseArea province = areaMapper.selectById(city.getParentId());
        AssertUtil.isNull(city, "省份不存在");

        Long caseNumber = caseNumberMapper.getCaseNumber(cityId);
        return new StringBuilder().append(province.getName()).append(city.getName()).append("#").append(caseNumber).toString();
    }

}
