package com.abi.qms.platform.service.impl;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.abi.qms.platform.dao.entity.InspectionCase;
import com.abi.qms.platform.dao.entity.InspectionCaseBoxCode;
import com.abi.qms.platform.dao.entity.InspectionCaseFlow;
import com.abi.qms.platform.dao.enums.CaseNumberGenerateStrategyEnum;
import com.abi.qms.platform.dao.enums.ReqChannelTypeEnum;
import com.abi.qms.platform.dao.mapper.InspectionCaseBoxCodeMapper;
import com.abi.qms.platform.dao.mapper.InspectionCaseFlowMapper;
import com.abi.qms.platform.dao.mapper.InspectionCaseMapper;
import com.abi.qms.platform.dao.vo.result.InspectionCaseDetailVO;
import com.abi.qms.platform.dao.vo.result.ListInspectionCaseVO;
import com.abi.qms.platform.dao.vo.result.ListRoleVO;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.service.FlowQueryService;
import com.abi.qms.platform.service.IInspectionCaseService;
import com.abi.qms.platform.service.InspectionCaseFlowService;
import com.abi.qms.platform.service.InspectionCaseNumberService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.models.auth.In;
import jodd.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Consumer;
import java.util.function.Predicate;
import java.util.stream.Collectors;


/**
 * 稽查案件 Service业务层处理
 *
 * @author ZhangZhao
 * @date: 2021-07-19
 */
@Service
public class InspectionCaseServiceImpl implements IInspectionCaseService {


    @Autowired
    private InspectionCaseMapper inspectionCaseMapper;

    @Autowired
    private InspectionCaseBoxCodeMapper inspectionCaseBoxCodeMapper;

    @Autowired
    private InspectionCaseFlowMapper inspectionCaseFlowMapper;

    @Autowired
    private InspectionCaseNumberService caseNumberService;

    @Autowired
    private FlowQueryService flowQueryService;


    /**
     * 新增案件
     *
     * @param addReq 新增请求
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addInspectionCase(AddInspectionCaseReq addReq) {

        //============先校验箱码============
        List<AddInspectionCaseReq.BoxCode> involvedBoxCodes = addReq.getInvolvedBoxCodes();
        List<String> boxCodeStrList = involvedBoxCodes.stream().map(AddInspectionCaseReq.BoxCode::getBoxCode).collect(Collectors.toList());
        //查询箱码流向信息
        List<GetBoxCodeRes> flowInfoByBoxCodeList = this.getFlowInfoByBoxCodeList(boxCodeStrList);
        //校验箱码信息
        this.validatedBoxCode(flowInfoByBoxCodeList, addReq.getSkuCode());
        //流向信息
        List<GetBoxCodeRes.FlowBean> flowList = flowInfoByBoxCodeList.get(0).getFlowList();

        //============案件编号============
        String caseNumber = addReq.getCaseNumber();
        //生成
        Integer generateStrategy = addReq.getCaseNumberGenerateStrategy();
        if (CaseNumberGenerateStrategyEnum.AUTO.is(generateStrategy)) {
            caseNumber = this.autoGenerateCaseNumber(addReq.getProvinceId(), addReq.getCityId());
        }

        //查看编号是否存在
        QueryWrapper<InspectionCase> inspectionCaseQW = new QueryWrapper<>();
        inspectionCaseQW.eq("case_number", caseNumber);
        List<InspectionCase> inspectionCaseList = inspectionCaseMapper.selectList(inspectionCaseQW);
        AssertUtil.isEmpty(inspectionCaseList, "当前编号已存在！");

        //查看案件名称是否存在
        inspectionCaseQW.clear();
        inspectionCaseQW.eq("case_name", addReq.getCaseName());
        inspectionCaseList = inspectionCaseMapper.selectList(inspectionCaseQW);
        AssertUtil.isEmpty(inspectionCaseList, "当前名称已占用！");

        //============保存案件============

        InspectionCase inspectionCase = new InspectionCase();
        BeanUtil.copyProperties(addReq, inspectionCase);
        //流出经销商 从 flowInfo 中取出
        GetBoxCodeRes.FlowBean outAgencyFlowBean = flowList.stream()
                .filter(s -> s.getAbnormalType().equals(2))
                .min(Comparator.comparing(GetBoxCodeRes.FlowBean::getOutStorageTime))
                .orElseThrow(() -> new BusinessException("没有流出异常节点"));

        inspectionCase.setOutAgencyCode(outAgencyFlowBean.getNodeId().toString());
        inspectionCase.setOutAgencyName(outAgencyFlowBean.getNodeName());

        // 设置流入 经销商 从 flowInfo 中取出
        GetBoxCodeRes.FlowBean inAgencyflowBean = flowList.stream()
                .filter(s -> s.getAbnormalType().equals(1))
                .max(Comparator.comparing(GetBoxCodeRes.FlowBean::getInStorageTime))
                .orElseThrow(() -> new BusinessException("没有流入异常节点"));

        inspectionCase.setInAgencyCode(inAgencyflowBean.getNodeId().toString());
        inspectionCase.setInAgencyName(inAgencyflowBean.getNodeName());
        //流货数量
        inspectionCase.setBoxCodeCount((long) addReq.getInvolvedBoxCodes().size());
        //案件名称
        inspectionCase.setCaseNumber(caseNumber);
        //设置渠道=>后台
        inspectionCase.setChannel(ReqChannelTypeEnum.BACKSTAGE.getCode());
        inspectionCaseMapper.insert(inspectionCase);


        //============保存涉案箱码============
        //转换类型
        List<InspectionCaseBoxCode> inspectionCaseBoxCodeList = involvedBoxCodes.stream()
                .parallel().map(s -> {
                    InspectionCaseBoxCode inspectionCaseBoxCode = new InspectionCaseBoxCode();
                    BeanUtil.copyProperties(s, inspectionCaseBoxCode);
                    inspectionCaseBoxCode.setInspectionCaseId(inspectionCase.getId());
                    return inspectionCaseBoxCode;
                }).collect(Collectors.toList());
        //批量保存
        inspectionCaseBoxCodeMapper.insertBatch(inspectionCaseBoxCodeList);


        //============保存【案件-流向】关联============

        List<InspectionCaseFlow> inspectionCaseFlowList = flowList.stream().map(s -> {
            InspectionCaseFlow inspectionCaseFlow = new InspectionCaseFlow();
            BeanUtil.copyProperties(s, inspectionCaseFlow);
            inspectionCaseFlow.setInspectionCaseId(inspectionCase.getId());
            return inspectionCaseFlow;
        }).collect(Collectors.toList());

        inspectionCaseFlowMapper.insertBatch(inspectionCaseFlowList);


    }


    /**
     * 分页列表查询
     *
     * @param pageListReq 查询条件和分页信息
     * @return 查询结果
     */
    @Override
    public PageListInspectionCaseRes pageListQuery(PageListInspectionCaseReq pageListReq) {
        IPage<ListInspectionCaseVO> pageResult = inspectionCaseMapper.pageListQuery(PageUtil.createPage(pageListReq), pageListReq);
        List<ListInspectionCaseVO> inspectionCaseVOList = pageResult.getRecords();

        //转换类型
        List<PageListInspectionCaseRes.InspectionCaseBean> inspectionCaseBeanList = inspectionCaseVOList.stream()
                .parallel().map(s -> {
                    PageListInspectionCaseRes.InspectionCaseBean inspectionCaseBean = new PageListInspectionCaseRes.InspectionCaseBean();
                    BeanUtil.copyProperties(s, inspectionCaseBean);
                    //经销商
                    inspectionCaseBean.setOutAgency(s.getOutAgencyCode() + s.getOutAgencyName());
                    inspectionCaseBean.setInAgency(s.getInAgencyCode() + s.getInAgencyName());
                    //渠道
                    inspectionCaseBean.setChannel(ReqChannelTypeEnum.getName(s.getChannel()));
                    return inspectionCaseBean;
                }).collect(Collectors.toList());


        //封装出参
        PageListInspectionCaseRes pageListInspectionCaseRes = new PageListInspectionCaseRes();
        //放入分页信息
        PageUtil.copyPageInfo(pageListInspectionCaseRes, pageResult);
        pageListInspectionCaseRes.setInspectionCaseList(inspectionCaseBeanList);
        return pageListInspectionCaseRes;
    }

    /**
     * 编辑案件
     *
     * @param editReq 编辑内容
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void editInspectionCase(EditInspectionCaseReq editReq) {

        final Long caseId = editReq.getId();

        InspectionCase inspectionCase = inspectionCaseMapper.selectById(caseId);
        //============校验箱码============
        //涉案箱码
        List<EditInspectionCaseReq.BoxCode> involvedBoxCodes = editReq.getInvolvedBoxCodes();
        List<String> boxCodeStrList = involvedBoxCodes.stream().map(EditInspectionCaseReq.BoxCode::getBoxCode).collect(Collectors.toList());
        //查询箱码流向信息
        List<GetBoxCodeRes> flowInfoByBoxCodeList = this.getFlowInfoByBoxCodeList(boxCodeStrList);
        //校验箱码信息
        this.validatedBoxCode(flowInfoByBoxCodeList, inspectionCase.getSkuCode());
        //流向信息
        List<GetBoxCodeRes.FlowBean> flowList = flowInfoByBoxCodeList.get(0).getFlowList();

        //============修改案件信息============
        //流出经销商 从 flowInfo 中取出
        GetBoxCodeRes.FlowBean outAgencyFlowBean = flowList.stream()
                .filter(s -> s.getAbnormalType().equals(2))
                .min(Comparator.comparing(GetBoxCodeRes.FlowBean::getOutStorageTime))
                .orElseThrow(() -> new BusinessException("没有流出异常节点"));

        inspectionCase.setOutAgencyCode(outAgencyFlowBean.getNodeId().toString());
        inspectionCase.setOutAgencyName(outAgencyFlowBean.getNodeName());

        // 设置流入 经销商 从 flowInfo 中取出
        GetBoxCodeRes.FlowBean inAgencyflowBean = flowList.stream()
                .filter(s -> s.getAbnormalType().equals(1))
                .max(Comparator.comparing(GetBoxCodeRes.FlowBean::getInStorageTime))
                .orElseThrow(() -> new BusinessException("没有流入异常节点"));

        inspectionCase.setInAgencyCode(inAgencyflowBean.getNodeId().toString());
        inspectionCase.setInAgencyName(inAgencyflowBean.getNodeName());

        inspectionCase.setBoxCodeCount((long) involvedBoxCodes.size());
        inspectionCase.setCaseRemark(editReq.getCaseRemark());

        int i = inspectionCaseMapper.updateById(inspectionCase);
        //判断当前案件是否存在 i如果大于0表示 更新成功  否则表示未找到指定数据
        AssertUtil.isMeets(i, x -> x > 0, "当前案件已经不存在！");

        //============ 修改关联的箱码信息 ============
        //转换类型
        List<InspectionCaseBoxCode> inspectionCaseBoxCodeList = involvedBoxCodes.stream()
                .parallel().map(s -> {
                    InspectionCaseBoxCode inspectionCaseBoxCode = new InspectionCaseBoxCode();
                    BeanUtil.copyProperties(s, inspectionCaseBoxCode);
                    inspectionCaseBoxCode.setInspectionCaseId(caseId);
                    return inspectionCaseBoxCode;
                }).collect(Collectors.toList());

        //先删除当前案件下关联的箱码信息
        QueryWrapper<InspectionCaseBoxCode> boxCodeDeleteWrapper = new QueryWrapper<>();
        boxCodeDeleteWrapper.eq("inspection_case_id", caseId);
        inspectionCaseBoxCodeMapper.delete(boxCodeDeleteWrapper);
        //批量保存
        inspectionCaseBoxCodeMapper.insertBatch(inspectionCaseBoxCodeList);


        //============修改【案件-流向】关联============

        //先删除当前案件下关联流向信息
        QueryWrapper<InspectionCaseFlow> inspectionCaseFlowQW = new QueryWrapper<>();
        inspectionCaseFlowQW.eq("inspection_case_id", caseId);
        inspectionCaseFlowMapper.delete(inspectionCaseFlowQW);

        //保存新信息
        List<InspectionCaseFlow> inspectionCaseFlowList = flowList.stream().map(s -> {
            InspectionCaseFlow inspectionCaseFlow = new InspectionCaseFlow();
            BeanUtil.copyProperties(s, inspectionCaseFlow);
            inspectionCaseFlow.setInspectionCaseId(inspectionCase.getId());
            return inspectionCaseFlow;
        }).collect(Collectors.toList());

        inspectionCaseFlowMapper.insertBatch(inspectionCaseFlowList);


    }

    /**
     * 删除指定案件
     *
     * @param ids 指定案件ids
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeInspectionCase(Long[] ids) {
        //先删除案件
        inspectionCaseMapper.deleteBatchIds(Arrays.asList(ids));
        //删除关联的箱码信息
        inspectionCaseBoxCodeMapper.deleteBatchCaseIds(ids);
        //删除【案件-流向】关联信息
        inspectionCaseFlowMapper.deleteBatchCaseIds(ids);

    }

    /**
     * 获取案件详情信息
     *
     * @param id ID
     * @return 案件详情信息
     */
    @Override
    public GetInspectionCaseInfoRes getInspectionCaseInfo(final Long id) {
        //案件信息
        InspectionCaseDetailVO caseDetailVO = inspectionCaseMapper.selectInspectionCaseDetailById(id);
        AssertUtil.isNull(caseDetailVO, "此案件未查询到！");

        GetInspectionCaseInfoRes getInspectionCaseInfoRes = new GetInspectionCaseInfoRes();
        BeanUtil.copyProperties(caseDetailVO, getInspectionCaseInfoRes);

        getInspectionCaseInfoRes.setOutAgency(caseDetailVO.getOutAgencyCode() + caseDetailVO.getOutAgencyName());
        getInspectionCaseInfoRes.setInAgency(caseDetailVO.getInAgencyCode() + caseDetailVO.getInAgencyName());

        //查询对应流向信息

        List<InspectionCaseFlow> inspectionCaseFlowList = inspectionCaseFlowMapper.getList(id);
        //转换类型
        List<GetInspectionCaseInfoRes.FlowBean> flowBeanList = PojoConverterUtils.copyList(inspectionCaseFlowList, GetInspectionCaseInfoRes.FlowBean.class);
        getInspectionCaseInfoRes.setFlowList(flowBeanList);

        return getInspectionCaseInfoRes;
    }

    /**
     * 根据 编号或名称 模糊查询
     *
     * @param numOrName 编号或名称
     * @return 案件信息
     */
    @Override
    public QueryCaseLikeNumOrNameRes queryCaseLikeNumOrName(final String numOrName) {
        //模糊查询
        QueryWrapper<InspectionCase> caseQueryWrapper = new QueryWrapper<>();
        caseQueryWrapper.like("case_number", numOrName).or().like("case_name", numOrName);
        List<InspectionCase> inspectionCases = inspectionCaseMapper.selectList(caseQueryWrapper);
        if (CollectionUtil.isEmpty(inspectionCases)) {
            return null;
        }
        QueryCaseLikeNumOrNameRes res = new QueryCaseLikeNumOrNameRes();
        //类型转换
        List<QueryCaseLikeNumOrNameRes.CaseBean> caseBeanList = inspectionCases.stream().map(s -> {
            QueryCaseLikeNumOrNameRes.CaseBean caseBean = new QueryCaseLikeNumOrNameRes.CaseBean();
            caseBean.setCaseId(s.getId());
            caseBean.setCaseNum(s.getCaseNumber());
            caseBean.setCaseName(s.getCaseName());
            return caseBean;
        }).collect(Collectors.toList());
        res.setCaseInfoList(caseBeanList);

        return res;
    }


    /**
     * 关联到已有案件
     *
     * @param req 请求
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void associatedWithCase(AssociatedWithCaseReq req) {

        Long caseId = req.getCaseId();
        String boxCode = req.getBoxCode();
        //查询箱码信息
        List<String> boxCodeList = Collections.singletonList(boxCode);
        List<GetBoxCodeRes> flowInfoByBoxCodeList = this.getFlowInfoByBoxCodeList(boxCodeList);
        GetBoxCodeRes getBoxCodeRes = flowInfoByBoxCodeList.get(0);

        List<GetBoxCodeRes.FlowBean> reqFlowList = getBoxCodeRes.getFlowList();
        //验证指定案件是否存在
        InspectionCase inspectionCase = inspectionCaseMapper.selectById(caseId);
        AssertUtil.isNull(inspectionCase, "指定案件不存在或已被删除！");

        //验证SKU是否一致
        String skuCode = inspectionCase.getSkuCode();
        AssertUtil.isMeets(getBoxCodeRes.getSku().getSkuId(), s -> StrUtil.equals(s, skuCode), "当前箱码与指定案件SKU不一致！");

        //验证当前箱码是否已经存在
        QueryWrapper<InspectionCaseBoxCode> boxCodeQW = new QueryWrapper<>();
        boxCodeQW.eq("inspection_case_id", caseId);
        List<InspectionCaseBoxCode> caseBoxCodes = inspectionCaseBoxCodeMapper.selectList(boxCodeQW);
        long count = caseBoxCodes.stream().filter(s -> StrUtil.equals(s.getBoxCode(), boxCode)).count();
        AssertUtil.isMeets(count, x -> x >= 0L, "指定案件已经关联当前箱码！");

        //验证当前箱码是否与案件流向一致
        QueryWrapper<InspectionCaseFlow> flowQW = new QueryWrapper<>();
        flowQW.eq("inspection_case_id", caseId);
        List<InspectionCaseFlow> flowList = inspectionCaseFlowMapper.selectList(flowQW);
        //当前案件流向节点
        List<Long> nodeIds = flowList.stream()
                .map(InspectionCaseFlow::getNodeId)
                .collect(Collectors.toList());
        //请求流向节点
        List<Long> reqNodeIds = reqFlowList.stream()
                .map(GetBoxCodeRes.FlowBean::getNodeId)
                .collect(Collectors.toList());


        //节点数量是否一致
        AssertUtil.isMeets(reqNodeIds.size(), size -> size == nodeIds.size(), "当前箱码与案件流向不一致！");
        //节点是否相同
        AssertUtil.isMeets(reqNodeIds, s -> CollectionUtil.containsAll(s, nodeIds), "当前箱码与案件流向不一致！");


        //当前案件最小出库时间
        LocalDateTime outStorageTime = flowList.stream()
                .filter(s -> Objects.nonNull(s.getOutStorageTime()))
                .min(Comparator.comparing(InspectionCaseFlow::getOutStorageTime))
                .get()
                .getOutStorageTime();

        //请求流向最小出库时间
        LocalDateTime resOutStorageTime = reqFlowList.stream()
                .filter(s -> Objects.nonNull(s.getOutStorageTime()))
                .min(Comparator.comparing(GetBoxCodeRes.FlowBean::getOutStorageTime))
                .get()
                .getOutStorageTime();

        //判断是否为同一批货  出库时间是否一致
        AssertUtil.isMeets(resOutStorageTime, s -> s.compareTo(outStorageTime) == 0, "当前箱码与案件流向不一致！");

        //修改案件信息
        Long boxCodeCount = inspectionCase.getBoxCodeCount();
        inspectionCase.setBoxCodeCount(++boxCodeCount);
        inspectionCaseMapper.updateById(inspectionCase);
        //添加箱码
        InspectionCaseBoxCode entity = new InspectionCaseBoxCode();
        entity.setBoxCode(boxCode);
        entity.setInspectionCaseId(caseId);
        inspectionCaseBoxCodeMapper.insert(entity);

    }

    /**
     * 查询涉案箱码列表
     *
     * @param caseId 案件ID
     * @return 涉案箱码列表
     */
    @Override
    public GetCaseBoxCodeListRes getBoxCodeListByCaseId(final Long caseId) {
        QueryWrapper<InspectionCaseBoxCode> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("inspection_case_id", caseId);
        List<InspectionCaseBoxCode> caseBoxCodes = inspectionCaseBoxCodeMapper.selectList(queryWrapper);
        AssertUtil.isNotEmpty(caseBoxCodes, "未查询到指定案件涉案箱码！");
        List<GetCaseBoxCodeListRes.BoxCodeBean> boxCodeBeans = PojoConverterUtils.copyList(caseBoxCodes, GetCaseBoxCodeListRes.BoxCodeBean.class);
        GetCaseBoxCodeListRes res = new GetCaseBoxCodeListRes();
        res.setBoxCodeList(boxCodeBeans);
        return res;
    }

    /**
     * 导出数据
     *
     * @param req 查询条件
     * @return
     */
    @Override
    public List<ExportInspectionCaseRes> export(ExportInspectionCaseReq req) {

        List<ListInspectionCaseVO> listInspectionCaseVOS = inspectionCaseMapper.queryList(req);

        //转换类型
        List<ExportInspectionCaseRes> exportDataList = listInspectionCaseVOS.stream()
                .parallel().map(s -> {
                    ExportInspectionCaseRes res = new ExportInspectionCaseRes();
                    BeanUtil.copyProperties(s, res);
                    //经销商
                    res.setOutAgency(s.getOutAgencyCode() + s.getOutAgencyName());
                    res.setInAgency(s.getInAgencyCode() + s.getInAgencyName());
                    //渠道
                    res.setChannel(ReqChannelTypeEnum.getName(s.getChannel()));
                    return res;
                }).collect(Collectors.toList());

        return exportDataList;
    }


    /**
     * 查询箱码对应流向信息
     *
     * @param boxCodeList 箱码列表
     * @return 对应流向信息列表
     */
    private List<GetBoxCodeRes> getFlowInfoByBoxCodeList(List<String> boxCodeList) {
        List<GetBoxCodeRes> getBoxCodeResList = new ArrayList<>();
        boxCodeList.forEach(s -> {
            GetBoxCodeRes res = flowQueryService.getFlowInfoByBoxCode(s);
            AssertUtil.isNull(res, "失败！箱码：" + s + "，失败原因：未查询到此箱码信息！");
            getBoxCodeResList.add(res);
        });
        return getBoxCodeResList;
    }


    /**
     * 校验箱码
     * 1、提交箱码的SKU信息是否一致，
     * 2、提交箱码的流向信息是否一致
     * 不一致则显示错误箱码；并列出错误原因
     *
     * @param boxCodeResList 箱码信息
     * @return 流向信息
     */
    private void validatedBoxCode(List<GetBoxCodeRes> boxCodeResList, String skuCode) {

        List<String> skuErrBoxCode = new ArrayList<>();
        List<String> flowErrBoxCode = new ArrayList<>();

        Set<LocalDateTime> outStorageTimeSet = new HashSet<LocalDateTime>();
        List<Long> firstFlowNodeIdList = new ArrayList<>();

        for (GetBoxCodeRes res : boxCodeResList) {

            //箱码
            String boxCode = res.getBoxStack().getBoxCode();

            //判断SKU信息是否一致
            String skuId = res.getSku().getSkuId();
            if (!StrUtil.equals(skuId, skuCode)) {
                skuErrBoxCode.add(boxCode);
            }

            //判断流向是否一致
            //获取当前流向信息
            List<GetBoxCodeRes.FlowBean> flowList = res.getFlowList();
            LocalDateTime outStorageTime = flowList.stream().filter(s -> Objects.nonNull(s.getOutStorageTime())).min(Comparator.comparing(GetBoxCodeRes.FlowBean::getOutStorageTime)).get().getOutStorageTime();
            boolean outStorageTimeAdd = outStorageTimeSet.add(outStorageTime);

            //获取当前所以流向节点ID
            List<Long> nodeIds = flowList.stream().map(GetBoxCodeRes.FlowBean::getNodeId).collect(Collectors.toList());
            //如果firstFlowNodeIdList 为空 则赋值
            if (CollectionUtil.isEmpty(firstFlowNodeIdList)) {
                firstFlowNodeIdList.addAll(nodeIds);
                continue;
            }
            //先判断节点数量
            if (firstFlowNodeIdList.size() != nodeIds.size()) {
                flowErrBoxCode.add(boxCode);
                break;
            }

            //判断节点信息和时间信息是否一致
            if (!CollectionUtil.containsAll(firstFlowNodeIdList, nodeIds) || outStorageTimeAdd) {
                flowErrBoxCode.add(boxCode);
            }
        }

        AssertUtil.isEmpty(skuErrBoxCode, "箱码：" + CollectionUtil.join(skuErrBoxCode, ";") + "<br/>失败原因：SKU信息不一致");
        AssertUtil.isEmpty(flowErrBoxCode, "箱码：" + CollectionUtil.join(flowErrBoxCode, ";") + "<br/>失败原因：流向信息不一致");
    }


    /**
     * 自动生成案件编号
     *
     * @param provinceId 省份ID
     * @param cityId     城市ID
     * @return 生成的编号
     */
    private String autoGenerateCaseNumber(Long provinceId, Long cityId) {
        AssertUtil.isNull(provinceId, "省份ID不能为空");
        AssertUtil.isNull(cityId, "城市ID不能为空");
        return caseNumberService.getCaseNumber(cityId);
    }


}
