package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.BaseFactory;
import com.abi.qms.platform.dao.entity.QrBoxMapping;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.enums.BoxMappingActiveStatusEnum;
import com.abi.qms.platform.dao.enums.InvalidEnum;
import com.abi.qms.platform.dao.mapper.BaseFactoryMapper;
import com.abi.qms.platform.dao.mapper.QrBoxMappingMapper;
import com.abi.qms.platform.dao.tablestore.entity.QrCode;
import com.abi.qms.platform.dao.vo.result.*;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.AsyncTableStoreUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.TokenUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.QrBoxMappingService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.excel.ExcelInputFactory;
import com.abi.task.common.excel.ExcelOutputFactory;
import com.abi.task.common.excel.preperties.QrBoxExportProperty;
import com.abi.task.common.tablestore.TableStorePlusUtils;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.google.common.collect.Maps;
import com.sun.org.apache.bcel.internal.generic.NEW;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author: fangxinjian
 * @date: 2021/05/27 11:19
 * @description:
 */
@Service
@Slf4j
public class QrBoxMappingServiceImpl implements QrBoxMappingService {

    @Autowired
    private QrBoxMappingMapper qrBoxMappingMapper;

    @Autowired
    private TableStorePlusUtils tableStorePlusUtils;

    @Autowired
    private QrBoxMappingService qrBoxMappingService;

    @Autowired
    private UserUtil userUtil;

    @Autowired
    private BaseFactoryMapper baseFactoryMapper;

    @Autowired
    private AsyncTableStoreUtil asyncTableStoreUtil;

    private static final String SUCCESS = "success";

    private static final String FAIL = "fail";

    /**
     * 生成箱码合一文件,罐码使用这个逻辑（用首位码段生成）
     * @param req
     * @return
     */
    @Override
    public String addQrBoxMappingByFirstAndLast(GenerateBarCodeReq req) {

        //获取登录用户信息
        UserInfo user = userUtil.getUser();

        //获取起始序号和结束序号
        Long indexBegin = getCodeIndex(req.getBeginQrCode());
        AssertUtil.isNull(indexBegin,"起始序号不存在！");
        Long indexEnd = getCodeIndex(req.getEndQrCode());
        AssertUtil.isNull(indexBegin,"结束序号不存在！");
        //条数
        Long qrCodeCount = indexEnd - indexBegin+1;
        if (qrCodeCount < 0) {
            throw new BusinessException("起始序号大于结束序号,无法生成条形码");
        }
        //判断至少开始结束序号相同的箱码合一不能存在
        QueryWrapper<QrBoxMapping> boxMappingQw = new QueryWrapper<>();
        boxMappingQw.eq("begin_qr_code",req.getBeginQrCode());
        boxMappingQw.eq("end_qr_code",req.getEndQrCode());
        boxMappingQw.eq("is_delete",0);
        List<QrBoxMapping> qrBoxMappingList = qrBoxMappingMapper.selectList(boxMappingQw);
        if(CollectionUtils.isNotEmpty(qrBoxMappingList)){
            throw new BusinessException("不能重复创建箱码合一数据");
        }

        //构造持久化对象
        QrBoxMapping qbm = new QrBoxMapping();
        qbm.setBoxCode(String.valueOf(IdWorker.getId()));
        qbm.setIndexBegin(indexBegin);
        qbm.setIndexEnd(indexEnd);
        qbm.setBeginQrCode(req.getBeginQrCode());
        qbm.setEndQrCode(req.getEndQrCode());
        qbm.setQrCodeCount(qrCodeCount);
        qbm.setActiveStatus(BoxMappingActiveStatusEnum.NOT_ACTIVE.getCode());
        qbm.setProjectNo(req.getProjectNo());
        qbm.setBoardNo(req.getBoardNo());
        qbm.setRemark(req.getRemark());
        qbm.setCreateUserName(user.getUserName());
        qbm.setCreateBy(user.getId());
        qbm.setPackageId(req.getPackageId());
        qbm.setInvalid(InvalidEnum.NOT_INVALID.getCode());
        qrBoxMappingMapper.insert(qbm);

        return qbm.getBoxCode();

    }

    /**
     * 生成箱码合一文件,盖码使用这个逻辑（每一个码都再tablestore记录箱码合一信息）
     * @param codeList
     * @return
     */
    @Override
    public String addQrBoxMappingByFileUpload(List<String> codeList,Long packageId) {
        //如果入参有问题则短路
        if(CollectionUtils.isEmpty(codeList)){
            throw new BusinessException("箱码合一的码数量为0");
        }

        //获取登录用户信息
        UserInfo user = userUtil.getUser();

        //1-构造持久化对象
        QrBoxMapping qrBoxMapping = new QrBoxMapping();
        qrBoxMapping.setBoxCode(String.valueOf(IdWorker.getId()));
        qrBoxMapping.setIndexBegin(0L);
        qrBoxMapping.setIndexEnd(0L);
        qrBoxMapping.setBeginQrCode(codeList.get(0));
        qrBoxMapping.setEndQrCode(codeList.get(codeList.size()-1));
        qrBoxMapping.setQrCodeCount(Long.valueOf(codeList.size()));
        qrBoxMapping.setActiveStatus(BoxMappingActiveStatusEnum.NOT_ACTIVE.getCode());
        qrBoxMapping.setCreateUserName(user.getUserName());
        qrBoxMapping.setCreateBy(user.getId());
        qrBoxMapping.setPackageId(packageId);
        qrBoxMapping.setInvalid(InvalidEnum.NOT_INVALID.getCode());
        qrBoxMappingMapper.insert(qrBoxMapping);

        //2-修改tablestore里面的码的箱码合一文件id
        Integer max = 10000;
        List<QrCode> qrCodeList = new ArrayList<>();
        for(int i=0;i<codeList.size();i++){
            QrCode qrCode = new QrCode();
            qrCode.setCode(codeList.get(i));
            qrCode.setQrBoxMappingId(qrBoxMapping.getId());
            qrCodeList.add(qrCode);
            //每一批满了 or 最后一批 就保存一下
            if((i+1)%max==0 || i==codeList.size()-1){
                asyncTableStoreUtil.mqUpdateRow(qrCodeList,"qrBoxMappingId");
                qrCodeList.clear();
            }
        }

        return qrBoxMapping.getBoxCode();
    }


    @Override
    public ListQrBoxCodeMappingRes listQrBoxCodeMapping(ListQrBoxCodeMappingReq req) {

        // 分页查询
        IPage<QrBoxMappingVO> iPage = qrBoxMappingMapper.listQrBoxCodeMapping(PageUtil.createPage(req), req);
        List<QrBoxMappingVO> qrboxMappingList = iPage.getRecords();

        // 封装出参、放入分页信息
        ListQrBoxCodeMappingRes res = new ListQrBoxCodeMappingRes();
        PageUtil.copyPageInfo(res, iPage);
        qrboxMappingList.forEach(item -> {
            if (item.getInvalid().equals(1)) {
                item.setActiveStatus(9);
            }
        });
        List<ListQrBoxCodeMappingRes.QrBoxMappingBean> beanList = PojoConverterUtils.copyList(qrboxMappingList, ListQrBoxCodeMappingRes.QrBoxMappingBean.class);
        res.setQrBoxMappingBeanList(beanList);

        return res;
    }

    @Override
    public QrBoxCodeUploadRes uploadCodeFiles(Map<String, MultipartFile> map, Long packageId) {

        QrBoxCodeUploadRes res = new QrBoxCodeUploadRes();
        Map<String, UploadFileRes> resultMap = Maps.newHashMap();
        map.entrySet().stream().forEach(o -> {
            MultipartFile file = o.getValue();
            //核心！！！上传文件
            try {
                //上传箱码文件
                doUpload(file, packageId);
                resultMap.put(o.getKey(), new UploadFileRes(SUCCESS, "上传箱码合一文件成功！"));
            } catch (BusinessException e) {
                log.warn("文件上传失败",e);
                throw e;
            } catch (Exception e) {
                log.warn("文件上传异常",e);
                throw new BusinessException(ErrorCodeEnum.FAIL.getCode(), "文件上传异常");
            }
        });

        res.put("data", resultMap);

        return res;
    }

    @Override
    public void activeBoxCode(ActiveBoxCodeReq req) {

        //获取登录用户信息
        UserInfo user = userUtil.getUser();

        QrBoxMapping qrBoxMapping = qrBoxMappingMapper.selectById(req.getId());
        AssertUtil.isNull(qrBoxMapping, "该箱码不存在！");

        qrBoxMapping.setActiveStatus(BoxMappingActiveStatusEnum.ACTIVATED.getCode());
        qrBoxMapping.setActiveTime(LocalDateTime.now());
        qrBoxMapping.setActiveUserId(user.getId());
        qrBoxMapping.setActiveUserName(user.getUserName());
        qrBoxMapping.setUpdateBy(user.getId());
        qrBoxMapping.setUpdateTime(LocalDateTime.now());

        qrBoxMappingMapper.updateById(qrBoxMapping);

    }

    @Override
    public PrintingDetailRes getPrintingDetail(PrintingDetailReq req) {
        PrintingDetailVO printingDetail = qrBoxMappingMapper.getPrintingDetail(req.getBoxCode());

        //构造出参
        PrintingDetailRes res = PojoConverterUtils.copy(printingDetail, PrintingDetailRes.class);
        res.setFactoryCoverName(null == printingDetail.getFactoryCoverLevelTwoId() ? "" : getFactoryName(printingDetail.getFactoryCoverLevelTwoId()));
        res.setFactoryBeerName(null == printingDetail.getFactoryBeerId() ? "" : getFactoryName(printingDetail.getFactoryBeerId()));
        return res;
    }

    @Override
    public void invalidBoxCode(ActiveBoxCodeReq req) {
        //获取登录用户信息
        UserInfo user = userUtil.getUser();

        QrBoxMapping qrBoxMapping = qrBoxMappingMapper.selectById(req.getId());
        AssertUtil.isNull(qrBoxMapping, "该箱码不存在！");

        qrBoxMapping.setInvalid(InvalidEnum.INVALID.getCode());
        qrBoxMapping.setUpdateBy(user.getId());
        qrBoxMapping.setUpdateTime(LocalDateTime.now());

        qrBoxMappingMapper.updateById(qrBoxMapping);
    }

    /**
     * 微信 - 箱码/条码详情
     *
     * @param queryWxQrBoxMappingDetailsWxReq
     * @return com.abi.qms.platform.dto.res.QueryWxQrBoxMappingDetailsWxRes
     * @author lu
     * @date 2021/6/9 11:24
     */
    @Override
    public QueryWxQrBoxMappingDetailsWxRes queryWxQrBoxMappingDetails(QueryWxQrBoxMappingDetailsWxReq queryWxQrBoxMappingDetailsWxReq) {
        QueryWxQrBoxMappingDetailsVO query = qrBoxMappingMapper.queryWxQrBoxMappingDetails(queryWxQrBoxMappingDetailsWxReq);
        AssertUtil.isNull(query, "条码序号查询为空");
        //封装出参
        return PojoConverterUtils.copy(query, QueryWxQrBoxMappingDetailsWxRes.class);
    }

    @Override
    public void replenishActivation(ReplenishActivationReq req) {
        QrBoxMapping qrBoxMapping = getActiveQrBoxMapping(req.getBoxCode());
        qrBoxMapping.setRemark(req.getReason());

        qrBoxMappingMapper.updateById(qrBoxMapping);

    }

    @Override
    public ActivationRecordRes getActivationRecord(ActivationRecordReq req) {

        // 是否小程序用户主账号 1-是 0-不是
        Integer isMainAccount = userUtil.getWholeUser().getIsMainAccount();

        ActivationRecordRes res = new ActivationRecordRes();
        Long id = TokenUtil.getUser().getId();
        if (isMainAccount.equals(1)) {
            id = null;
        }
        List<ActivationRecordVO> recordVOList = qrBoxMappingMapper.getActivationRecord(req,id);
        List<ActivationRecordRes.ActivationRecordBean> beanList = PojoConverterUtils.copyList(recordVOList, ActivationRecordRes.ActivationRecordBean.class);
        res.setActivationRecordBeanList(beanList);
        res.setIsMainAccount(isMainAccount);
        return res;
    }

    @Override
    public void activateNow(ActivateNowReq req) {
        List<String> boxCodeList = req.getBoxCodeList();
        for (String boxCode : boxCodeList) {
            QrBoxMapping qrBoxMapping = getActiveQrBoxMapping(boxCode);
            qrBoxMapping.setProductionLineName(req.getProductionLineName());
            qrBoxMapping.setProductionTime(req.getProductionTime());
            qrBoxMappingMapper.updateById(qrBoxMapping);
        }
    }

    @Override
    public ActivateDetailRes getActivateDetail(ActivateDetailReq req) {

        ActivateDetailVo activateDetailVo = qrBoxMappingMapper.getActivateDetail(req.getBoxCode());
        AssertUtil.isNull(activateDetailVo, "条码序号查询为空");

        ActivateDetailRes res = PojoConverterUtils.copy(activateDetailVo, ActivateDetailRes.class);
        return res;
    }

    @Override
    public List<QrBoxExportProperty> downloadExcelData(ListQrBoxCodeMappingExportReq req) {

        List<QrBoxMappingVO> qrBoxMappingVOList = qrBoxMappingMapper.listQrBoxCodeMappingForExcel(req.getActiveStatus(),req.getPackageId());

        List<QrBoxExportProperty> qrBoxExportProperties = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(qrBoxMappingVOList)){
            for (QrBoxMappingVO qrBoxMappingVO:qrBoxMappingVOList) {
                QrBoxExportProperty qrBoxExportProperty = PojoConverterUtils.copy(qrBoxMappingVO, QrBoxExportProperty.class);
                if(InvalidEnum.INVALID.is(qrBoxMappingVO.getInvalid())){
                    qrBoxExportProperty.setActiveStatus(InvalidEnum.INVALID.getName());
                }else{
                    if(BoxMappingActiveStatusEnum.NOT_ACTIVE.is(qrBoxMappingVO.getActiveStatus())){
                        qrBoxExportProperty.setActiveStatus(BoxMappingActiveStatusEnum.NOT_ACTIVE.getName());
                    }else if(BoxMappingActiveStatusEnum.ACTIVATED.is(qrBoxMappingVO.getActiveStatus())){
                        qrBoxExportProperty.setActiveStatus(BoxMappingActiveStatusEnum.ACTIVATED.getName());
                    }
                }
                qrBoxExportProperties.add(qrBoxExportProperty);
            }
        }

        return qrBoxExportProperties;
    }

    @Override
    public void downloadExcelData(ListQrBoxCodeMappingExportReq req, HttpServletResponse response) {
        List<QrBoxExportProperty>  properties = qrBoxMappingService.downloadExcelData(req.getActiveStatus(),req.getPackageId());
        //调用导出方法
        ExcelOutputFactory.outputStreamExcel("箱码合一", response, properties, QrBoxExportProperty.class);
    }

    @Override
    public List<QrBoxExportProperty> downloadExcelData(Integer activeStatus, Long packageId) {
        List<QrBoxMappingVO> qrBoxMappingVOList = qrBoxMappingMapper.listQrBoxCodeMappingForExcel(activeStatus,packageId);
        List<QrBoxExportProperty> qrBoxExportProperties = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(qrBoxMappingVOList)){
            for (QrBoxMappingVO qrBoxMappingVO:qrBoxMappingVOList) {
                QrBoxExportProperty qrBoxExportProperty = PojoConverterUtils.copy(qrBoxMappingVO, QrBoxExportProperty.class);
                if(InvalidEnum.INVALID.is(qrBoxMappingVO.getInvalid())){
                    qrBoxExportProperty.setActiveStatus(InvalidEnum.INVALID.getName());
                }else{
                    if(BoxMappingActiveStatusEnum.NOT_ACTIVE.is(qrBoxMappingVO.getActiveStatus())){
                        qrBoxExportProperty.setActiveStatus(BoxMappingActiveStatusEnum.NOT_ACTIVE.getName());
                    }else if(BoxMappingActiveStatusEnum.ACTIVATED.is(qrBoxMappingVO.getActiveStatus())){
                        qrBoxExportProperty.setActiveStatus(BoxMappingActiveStatusEnum.ACTIVATED.getName());
                    }
                }
                qrBoxExportProperties.add(qrBoxExportProperty);
            }
        }
        return qrBoxExportProperties;
    }

    /**
     * 组装激活公共所需参数
     *
     * @param boxCode
     * @return
     */
    private QrBoxMapping getActiveQrBoxMapping(String boxCode) {
        QueryWrapper<QrBoxMapping> qw = new QueryWrapper<>();
        qw.eq("box_code", boxCode);
        qw.eq("is_delete", 0);
        QrBoxMapping qrBoxMapping = qrBoxMappingMapper.selectOne(qw);
        AssertUtil.isNull(qrBoxMapping, "条码序号错误，未找到该条码序号！");
        qrBoxMapping.setActiveTime(LocalDateTime.now());
        qrBoxMapping.setActiveStatus(BoxMappingActiveStatusEnum.ACTIVATED.getCode());
        qrBoxMapping.setActiveUserId(userUtil.getUser().getId());
        qrBoxMapping.setActiveUserName(userUtil.getUser().getUserName());

        return qrBoxMapping;
    }


    /**
     * 根据主键查询已生成码包位置
     *
     * @param index 主键
     * @return
     */
    private Long getCodeIndex(String index) {
        //如果是URL去除前面的部分留下最后码
        index = index.substring(index.lastIndexOf("/") + 1);

        QrCode qrCode = tableStorePlusUtils.selectOne(QrCode.class, index);
        return qrCode.getCodeIndex();
    }

    public void doUpload(MultipartFile file, Long packageId) {
        if (file.isEmpty()) {
            throw new BusinessException("文件为空，请重新选择文件");
        }
        InputStream inputStream = null;
        LineNumberReader reader = null;
        String filename = file.getName();
        String encryptCodeStr;

        try {
            inputStream = file.getInputStream();
            //解决服务器上乱码
            reader = new LineNumberReader(new InputStreamReader(inputStream, "UTF-8"));
            //待修改的tablestore对象列表
            List<String> codeList = new ArrayList<>();

            //1-逐行读取文件的码
            while ((encryptCodeStr = reader.readLine()) != null) {
                log.debug("{} line : {}", filename, encryptCodeStr);
                if (StringUtils.isBlank(encryptCodeStr)) {
                    continue;
                }
                if (encryptCodeStr.contains(",")) {
                    throw new BusinessException("码包文件格式错误,建议每行一个码！");
                }
                //截取 去掉uri
                encryptCodeStr = encryptCodeStr.substring(encryptCodeStr.lastIndexOf("/") + 1);
                //放入tablestore entity的主键code
                codeList.add(encryptCodeStr);
            }

            //生成条形码
            addQrBoxMappingByFileUpload(codeList,packageId);

        } catch (BusinessException e) {
            throw e;
        } catch (Exception e) {
            log.warn("上传失败", e);
            throw new BusinessException("上传失败");
        } finally {
            //关闭流
            closeStream(inputStream, reader);
        }
    }

    private void closeStream(InputStream inputStream, BufferedReader reader) {
        try {
            if (null!=inputStream){
                inputStream.close();
            }
        } catch (Exception e) {
            log.warn("inputStream关闭失败");
            e.printStackTrace();
        }
        if (reader != null) {
            try {
                reader.close();
            } catch (Exception e) {
                log.warn("reader关闭失败");
                e.printStackTrace();
            }
        }
    }

    private String getFactoryName(Long id) {
        QueryWrapper<BaseFactory> qw = new QueryWrapper<>();
        qw.eq("id", id);
        qw.eq("is_delete", 0);
        BaseFactory baseFactory = baseFactoryMapper.selectOne(qw);
        return baseFactory.getFactoryName();
    }

}
